function MapExtension.EGUI.AttachDragFrame(parent, fTitle, variable, fLeft)
	local bindedFrame = parent
	if fTitle then
		bindedFrame = parent:GetBorder()
	end

	local frame = UI.CreateFrame("Frame", "", bindedFrame)
	frame:SetPoint("TOPLEFT", bindedFrame, "TOPLEFT", 0, 0)
	if fTitle then
		frame:SetPoint("BOTTOMRIGHT", bindedFrame, "TOPRIGHT", 0, parent:GetTop() - bindedFrame:GetTop())
	else
		frame:SetPoint("BOTTOMRIGHT", bindedFrame, "BOTTOMRIGHT", 0, 0)
	end
	frame:SetLayer(100)
	frame.parent = parent
	frame.MouseDown = false

	function frame.MouseButtonDown(fButton)
		if (fButton == fLeft) then
			frame.MouseDown = true
			frame.Drag = false
			local mouseData = Inspect.Mouse()
			frame.xDiff = frame.parent:GetLeft() - mouseData.x
			frame.yDiff = frame.parent:GetTop() - mouseData.y
			if frame.parent.Event.RightDown then frame.parent.Event.RightDown() end
		end
	end

	function frame.Event:LeftDown()
		self.MouseButtonDown(true)
		if self.parent.Event.LeftDown then self.parent.Event.LeftDown() end
	end

	function frame.Event:RightDown()
		self.MouseButtonDown(false)
		if self.parent.Event.RightDown then self.parent.Event.RightDown() end
	end
	
	function frame.Event:MouseMove(mouseX, mouseY)
		if self.MouseDown then
			self.Drag = true
			self.parent:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouseX + self.xDiff, mouseY + self.yDiff)
			MapExtensionUISettings[variable].x = mouseX + self.xDiff
			MapExtensionUISettings[variable].y = mouseY + self.yDiff
		end
		if self.parent.Event.MouseMove then self.parent.Event.MouseMove() end
	end
	
	function frame.Event:LeftUp()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.LeftUp then self.parent.Event.LeftUp() end
	end

	function frame.Event:LeftUpoutside()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.LeftUp then self.parent.Event.LeftUp() end
	end

	function frame.Event:RightUp()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.RightUp then self.parent.Event.RightUp() end
	end

	function frame.Event:RightUpoutside()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.RightUp then self.parent.Event.RightUp() end
	end

	function frame.Event:LeftClick()
		if not self.Drag and self.parent.Event.LeftClick then self.parent.Event.LeftClick() end
	end

	function frame.Event:MouseIn()
		if self.parent.Event.MouseIn then self.parent.Event.MouseIn() end
	end

	function frame.Event:MouseOut()
		if self.parent.Event.MouseOut then self.parent.Event.MouseOut() end
	end
	return frame
end

function MapExtension.EGUI.AttachResizableFrame(parent, variable, minWidth, minHeight)
	local frame = UI.CreateFrame("Frame", "", parent)
	frame:SetPoint("CENTER", parent, "BOTTOMRIGHT", 0, 0)
	frame:SetWidth(20)
	frame:SetHeight(20)

	frame.parent = parent
	frame.MouseDown = false

	function frame.Event:LeftDown()
		frame.MouseDown = true
		frame.mouseData = Inspect.Mouse()
		frame.width = frame.parent:GetWidth()
		frame.height = frame.parent:GetHeight()
	end

	function frame.Event:MouseMove(mouseX, mouseY)
		if self.MouseDown then
			local width, height = frame.width - self.mouseData.x + mouseX, frame.height - self.mouseData.y + mouseY
			self.parent:SetWidth(iif(width < minWidth, minWidth, width))
			self.parent:SetHeight(iif(height < minHeight, minHeight, height))
			MapExtensionUISettings[variable].width = self.parent:GetWidth()
			MapExtensionUISettings[variable].height = self.parent:GetHeight()
		end
	end
	
	function frame.Event:LeftUp()
		self.MouseDown = false
	end

	function frame.Event:LeftUpoutside()
		self.MouseDown = false
	end

	return frame
end

MapExtension.EGUI.TextureList = {}

function MapExtension.EGUI.SetTextureAsync(id, frame, source, texture)
	if MapExtension.EGUI.TextureList[id] == nil then MapExtension.EGUI.TextureList[id] = {} end
	table.insert(MapExtension.EGUI.TextureList[id], { frame, source, texture })
end

function MapExtension.EGUI.LoadAsyncTextures()
	if Inspect.System.Watchdog() < 0.1 then
		return
	end

--	while Inspect.System.Watchdog() > 0.1 and #MapExtension.EGUI.TextureList[MapExtension.AE.UI.MapTexture.active] > 0 and MapExtension.AE.Visible do
	if #MapExtension.EGUI.TextureList[MapExtension.AE.UI.MapTexture.active] > 0 and MapExtension.AE.Visible then
		local texture = table.remove(MapExtension.EGUI.TextureList[MapExtension.AE.UI.MapTexture.active], math.floor(math.random(#MapExtension.EGUI.TextureList[MapExtension.AE.UI.MapTexture.active])))
		texture[1]:SetTexture(texture[2], texture[3])
	end
	if #MapExtension.EGUI.TextureList[0] == 0 and #MapExtension.EGUI.TextureList[1] == 0 then
		Command.Event.Detach(Event.System.Update.End, MapExtension.EGUI.LoadAsyncTextures, "Event.System.Update.End - MapExtension.Event.SystemUpdateEnd")
	end
end
